@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,6 +53,7 @@ import java.util.ArrayList;
 import java.util.Hashtable;
 import java.util.List;
 import javax.mail.internet.AddressException;
+import javax.mail.internet.InternetAddress;
 import org.apache.jsieve.SieveException;
 import org.json.JSONArray;
 import org.json.JSONException;
@@ -60,10 +61,13 @@ import org.json.JSONObject;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.config.Filter;
 import com.openexchange.exception.OXException;
+import com.openexchange.java.Strings;
 import com.openexchange.jsieve.commands.ActionCommand;
 import com.openexchange.jsieve.commands.IfCommand;
 import com.openexchange.jsieve.commands.Rule;
+import com.openexchange.mail.json.parser.MessageParser;
 import com.openexchange.mail.mime.QuotedInternetAddress;
+import com.openexchange.mail.mime.utils.MimeMessageUtility;
 import com.openexchange.mail.utils.MailFolderUtility;
 import com.openexchange.mailfilter.MailFilterProperties;
 import com.openexchange.mailfilter.exceptions.MailFilterExceptionCode;
@@ -158,6 +162,37 @@ final class ActionCommandMapper implements Mapper<Rule> {
                 arrayList.add(Rule2JSON2Rule.createTagArg(VacationActionFields.SUBJECT));
                 arrayList.add(stringToList(subject));
             }
+            final String fromFieldName = VacationActionFields.FROM.getFieldname();
+            if (object.has(fromFieldName)) {
+                String from = "";
+                Object obj = object.get(fromFieldName);
+                boolean strict = true;
+                if (obj instanceof JSONArray) {
+                    // Create the object with the array of arrays as it is expected to be form the MessageParser.parseAddressKey
+                    try {
+                        InternetAddress[] fromArr = MessageParser.parseAdressArray(new JSONArray().put(obj), 1, strict);
+                        if (fromArr.length > 0) {
+                            from = fromArr[0].toString();
+                        }
+                    } catch (AddressException e) {
+                        throw OXJSONExceptionCodes.INVALID_VALUE.create(from, VacationActionFields.FROM.getFieldname());
+                    }
+                } else if (obj instanceof String) {
+                    // Get string
+                    String fromStr = object.getString(fromFieldName);
+                    try {
+                        new QuotedInternetAddress(fromStr, strict);
+                        from = fromStr;
+                    } catch (AddressException e) {
+                        throw OXJSONExceptionCodes.INVALID_VALUE.create(from, VacationActionFields.FROM.getFieldname());
+                    }
+                }
+                
+                if (!Strings.isEmpty(from)) {
+                    arrayList.add(Rule2JSON2Rule.createTagArg(VacationActionFields.FROM));
+                    arrayList.add(stringToList(from));
+                }
+            }
             final String text = object.getString(VacationActionFields.TEXT.getFieldname());
             if (null == text) {
                 throw OXJSONExceptionCodes.JSON_READ_ERROR.create("Parameter " + VacationActionFields.TEXT.getFieldname() + " is missing for " + ActionCommand.Commands.VACATION.getJsonname() + " is missing in JSON-Object. This is a required field");
@@ -301,6 +336,11 @@ final class ActionCommandMapper implements Mapper<Rule> {
                 if (null != subject) {
                     tmp.put(VacationActionFields.SUBJECT.getFieldname(), subject.get(0));
                 }
+                final List<String> from = tagarguments.get(VacationActionFields.FROM.getTagname());
+                if (null != from) {
+                    String decodedFrom = MimeMessageUtility.decodeEnvelopeSubject(from.get(0));
+                    tmp.put(VacationActionFields.FROM.getFieldname(), decodedFrom);
+                }
                 tmp.put(VacationActionFields.TEXT.getFieldname(), ((List<String>) arguments.get(arguments.size() - 1)).get(0));
             } else if (ActionCommand.Commands.ENOTIFY.equals(actionCommand.getCommand())) {
                 tmp.put(GeneralFields.ID, ActionCommand.Commands.ENOTIFY.getJsonname());
