@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2014 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -753,6 +753,7 @@ public final class SieveTextFilter {
         boolean newline = true;
         boolean comment = false;
         boolean commentremoved = false;
+        boolean inQuotes = false;
         for (int i = 0; i < sb.length();) {
             final char c = sb.charAt(i);
             if (c == '\n') {
@@ -768,6 +769,7 @@ public final class SieveTextFilter {
                 // The comment flag is important because otherwise
                 // you could struggle about '"' in comments
                 dontparse = !dontparse;
+                inQuotes = !inQuotes;
                 nextchar = false;
                 i++;
             } else if (!commentremoved && !comment && c == 't') {
@@ -786,7 +788,7 @@ public final class SieveTextFilter {
                 nextchar = true;
                 newline = false;
             } else if (dontparse && c == '.') {
-                if (sb.charAt(i - 1) == '\n' && sb.charAt(i + 1) == '\r') {
+                if (sb.charAt(i - 1) == '\n' && sb.charAt(i + 1) == '\r' && !inQuotes) {
                     dontparse = false;
                 }
                 nextchar = false;
